@@ -8,6 +8,7 @@ import itertools
 import more_itertools
 import pickle
 import warnings
+from collections.abc import Iterable
 
 from tqdm import tqdm
 
@@ -30,15 +31,14 @@ class BitrixSemaphoreWrapper():
     Используется для контроля скорости доступа к серверам Битрикс.
 
     Основная цель - вести учет количества запросов, которые можно передать
-    серверу Битрикс без получения ошибки 503.
+    серверу Битрикс без получения ошибки `503`.
 
     Используется как контекстный менеджер, оборачивающий несколько
     последовательных запросов к серверу.
 
-    Свойства:
-    - release_task - указатель на задачу, которая увеличивает счетчик количества
-        доступных запросов к серверу Битрикс. Должна выполняться параллельно
-        с другими задачами в цикле asyncio.
+    Чтобы все работало, нужно, чтобы внутри метода класса `Bitrix`, в котором
+    используется этот семафор, выполнял параллельно по совими задачами и
+    корутину-метод `release_sem()`.
         
     Параметры:
     - pool_size: int - размер пула доступных запросов.
@@ -46,6 +46,7 @@ class BitrixSemaphoreWrapper():
 
     Методы:
     - acquire(self)
+    - release_sem(self)
     '''
 
 
@@ -54,59 +55,71 @@ class BitrixSemaphoreWrapper():
         self._stopped_value = None
         self.requests_per_second = requests_per_second
         self._pool_size = pool_size
-        self._slow_last_acquire = None
 
     async def __aenter__(self):
         global _SLOW
         self._sem = asyncio.BoundedSemaphore(self._pool_size)
-        if (self._stopped_time) and not (_SLOW):
-            '''
+        if _SLOW:
+            self._slow_lock = asyncio.Lock()
+        else:
+            if self._stopped_time:
+                '''
 -----v-----------------------------v---------------------
      ^ - _stopped_time             ^ - current time
      |-------- time_passed --------|
      |- step -|- step -|- step |          - add_steps (whole steps to add)
                                |- step -| - additional 1 step added
                                    |-aw-| - additional waiting time
-            '''
-            time_passed = time.monotonic() - self._stopped_time
+                '''
+                time_passed = time.monotonic() - self._stopped_time
 
-            # сколько шагов должно было пройти
-            add_steps = time_passed / self.requests_per_second // 1
+                # сколько шагов должно было пройти
+                add_steps = time_passed / self.requests_per_second // 1
 
-            # сколько шагов могло пройти с учетом ограничений + еще один
-            real_add_steps = min(self._pool_size - self._stopped_value,
-                                 add_steps + 1)
+                # сколько шагов могло пройти с учетом ограничений + еще один
+                real_add_steps = min(self._pool_size - self._stopped_value,
+                                    add_steps + 1)
 
-            # добавляем пропущенные шаги
-            self._sem._value += real_add_steps
+                # добавляем пропущенные шаги
+                self._sem._value += real_add_steps
 
-            # ждем время, излишне списанное при добавлении дополнительного шага
-            await asyncio.sleep((add_steps + 1) / self.requests_per_second - time_passed)
+                # ждем время, излишне списанное при добавлении дополнительного шага
+                await asyncio.sleep((add_steps + 1) / self.requests_per_second - time_passed)
 
-            self._stopped_time = None
-            self._stopped_value = None
+                self._stopped_time = None
+                self._stopped_value = None
 
-        self.release_task = asyncio.create_task(self._release_sem())
 
     async def __aexit__(self, a1, a2, a3):
         self._stopped_time = time.monotonic()
         
-        # в slow-режиме обнуляем пул запросов, чтобы после выхода
-        # не выдать на сервер пачку запросов и не словить отказ
-        self._stopped_value = 0 if _SLOW else self._sem._value
-        self.release_task.cancel()
+        if _SLOW:
+            # в slow-режиме обнуляем пул запросов, чтобы после выхода
+            # не выдать на сервер пачку запросов и не словить отказ
+            self._stopped_value = 0
+        else:
+            self._stopped_value = self._sem._value
+
+
+    async def release_sem(self):
+        '''
+        Корутина-метод, которая увеличивает счетчик доступных в пуле запросов.
+
+        Должна запускаться единожды в параллели со всеми другими задачами
+        внутри основного цикла `Bitrix._request_list`, кроме случаев
+        выполнения в slow-режиме, когда она запускаться на должна.
+        '''
+
+        while True:
+            if self._sem._value < self._sem._bound_value:
+                self._sem.release()
+            await asyncio.sleep(1 / self.requests_per_second)
 
-    async def _release_sem(self):
-        if not _SLOW:
-            while True:
-                if self._sem._value < self._sem._bound_value:
-                    self._sem.release()
-                await asyncio.sleep(1 / self.requests_per_second)
 
     async def acquire(self):
         '''
-        Вызов acquire() должен предшествовать любому обращению
-        к серверу Bitrix. Он возвращает True, когда к серверу
+        Вызов `await acquire()` должен предшествовать любому обращению
+        к серверу Битрикс. Он возвращает `True`, когда к серверу
         можно осуществить запрос.
 
         Использование:
@@ -118,16 +131,10 @@ class BitrixSemaphoreWrapper():
         '''
         global _SLOW, _SLOW_RPS
         if _SLOW:
-            # ждать, исходя из времени последнего запроса в slow-режиме
-            await asyncio.sleep(
-                max (0, 1 / _SLOW_RPS - (time.monotonic() - self._slow_last_acquire))
-                if self._slow_last_acquire 
-                else 1 / _SLOW_RPS
-            ) 
-
-            # подготовиться к следующему вызову
-            self._slow_last_acquire = time.monotonic()
-
+            # ждать, пока отработают другие запросы, запущенные параллельно,
+            async with self._slow_lock:
+            # потом ждать основное время "остывания"
+                await asyncio.sleep(1 / _SLOW_RPS)
             return True 
         else:
             return await self._sem.acquire()
@@ -146,19 +153,15 @@ class Bitrix:
 
     Параметры:
     - webhook: str - URL вебхука, полученного от сервера Битрикс
-    
-    - autobatch: bool = True - автоматически объединять списки запросов в батчи
-
     - verbose: bool = True - показывать ли прогрессбар при выполнении запроса
 
     Методы:
     - get_all(self, method: str, details=None)
     - get_by_ID(self, method: str, ID_list, details=None)
-    - post(self, method: str, item_list)
-    - set_requests_per_second(self, requests_per_second: float)
+    - call(self, method: str, item_list)
     '''
 
-    def __init__(self, webhook: str, autobatch: bool = True, verbose: bool = True):
+    def __init__(self, webhook: str, verbose: bool = True):
         '''
         Создает объект класса Bitrix.
 
@@ -166,7 +169,7 @@ class Bitrix:
         
         self.webhook = _correct_webhook(webhook)
         self._sw = BitrixSemaphoreWrapper(BITRIX_POOL_SIZE, BITRIX_RPS)
-        self._autobatch = autobatch
+        self._autobatch = True
         self._verbose = verbose
 
 
@@ -209,13 +212,17 @@ class Bitrix:
             item_list = batches
 
         async with self._sw, aiohttp.ClientSession(raise_for_status=True) as session:
-            tasks = (asyncio.create_task(self._request(session, method, i))
-                     for i in item_list)
-            results = []
+            global _SLOW
+            tasks = [asyncio.create_task(self._request(session, method, i))
+                        for i in item_list]
+            if not _SLOW:
+                tasks.append(asyncio.create_task(self._sw.release_sem()))
+
             if self._verbose:
                 pbar = tqdm(total=real_len, initial=real_start)
+            results = []
             tasks_to_process = len(item_list)
-            for x in asyncio.as_completed((*tasks, self._sw.release_task)):
+            for x in asyncio.as_completed(tasks):
                 r, __ = await x
                 if r['result_error']:
                     raise RuntimeError(f'The server reply contained an error: {r["result_error"]}')
@@ -286,12 +293,12 @@ class Bitrix:
         if params:
             _check_params(params)
             for k in params.keys():
-                if k in ['start', 'limit', 'order']:
+                if k.lower() in ['start', 'limit', 'order']:
                     raise ValueError("get_all() doesn't support parameters 'start', 'limit' or 'order'")
 
         return asyncio.run(self._get_paginated_list(method, params))
 
-    def get_by_ID(self, method: str, ID_list, params=None):
+    def get_by_ID(self, method: str, ID_list: Iterable, params=None):
         '''
         Получить список сущностей по запросу method и списку ID.
 
@@ -319,7 +326,15 @@ class Bitrix:
         сущности.
         '''
 
-        if params: _check_params(params)
+        if params: 
+            _check_params(params)
+            for k in params.keys():
+                if k.lower() == 'id':
+                    raise ValueError("get_by_ID() doesn't support parameter 'ID' within the 'params' argument")
+
+        if not isinstance(ID_list, Iterable):
+            raise TypeError("get_by_ID(): 'ID_list' should be iterable")
+
         if len(ID_list) == 0:
             return []
         return asyncio.run(self._request_list(
@@ -329,7 +344,7 @@ class Bitrix:
             preserve_IDs=True
         ))
 
-    def call(self, method: str, item_list):
+    def call(self, method: str, item_list: Iterable):
         '''
         Вызвать метод REST API по списку.
 
@@ -348,6 +363,9 @@ class Bitrix:
             raise ValueError(
                 'item_list contains items with incorrect method params') from err 
 
+        if not isinstance(item_list, Iterable):
+            raise TypeError("get_by_ID(): 'item_list' should be iterable")
+
         return asyncio.run(self._request_list(method, item_list))
 
 
@@ -434,22 +452,23 @@ def _check_params(p):
         'order': dict,
         'filter': dict,
         'start': int,
-        'fields': dict,
-        'id': None
+        'fields': dict
     }
 
-    for pk in p.keys():
-        if pk.lower() not in clauses.keys():
-            raise ValueError(f'Unknown clause "{pk}" in params argument')
+#    for pk in p.keys():
+#        if pk.lower() not in clauses.keys():
+#            raise ValueError(f'Unknown clause "{pk}" in params argument')
 
     # check for allowed types of key values
     for pi in p.items():
-        t = clauses[pi[0].lower()]
-        if t and not (
-            (isinstance(pi[1], t)) or
-            ((t == list) and (any([isinstance(pi[1], x) for x in [list, tuple, set]])))
-        ):
-            raise TypeError(f'Clause "{pi[0]}" should be of type {t}, but its type is {type(pi[1])}')
+        if pi[0] in clauses.keys():
+            t = clauses[pi[0].lower()]
+            if t and not (
+                (isinstance(pi[1], t)) or
+                ((t == list) and (any([isinstance(pi[1], x) for x in [list, tuple, set]])))
+            ):
+                raise TypeError(f'Clause "{pi[0]}" should be of type {t}, '
+                    'but its type is {type(pi[1])}')
 
 
 def _url_valid(url):
